# * ====================================================================
# * Copyright (c) 1995,1996, ISIS/JRC.  All rights reserved.
# *
# * This product was developed within the Institute for Safety, Informatics
# * and Systems of the Joint Research Centre of the European Commission.
# *
# * More information about us; and this software product can
# * be found on http://www.jrc.it.
# * 
# * With this file you should have received a file called 'License'.
# * 
# * Redistribution and use in source and binary forms, with or without
# * modification, are permitted provided that the following conditions
# * are met:
# *
# * 1. Redistributions of source code must retain the above copyright
# *    notice, this list of conditions and the following disclaimer. 
# *
# * 2. Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in
# *    the documentation and/or other materials provided with the
# *    distribution.
# *
# * 3. All materials mentioning features or use of this
# *    software must display the following acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * 4. The name of the Copyright holder must not be used to
# *    endorse or promote products derived from this software without
# *    prior written permission.
# *
# * 5. Redistributions of any form whatsoever must retain the following
# *    acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * THIS SOFTWARE IS PROVIDED BY ISIS/JRC ``AS IS'' AND ANY
# * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ISIS/JRC OR
# * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# * OF THE POSSIBILITY OF SUCH DAMAGE.
# * ====================================================================
# Module Display
#
# Displays a record in various ways

# If you set $::NAMESTRING and $::VERSIONSTRING the module
# assumes that we are the web server ourselves and adds the necessary
# headers.

use strict;

use CGI qw(-no_debug);


# Configurable variables
#
$Display::HTML::admin_email = 'enrm-admin@jrc.it';
$Display::HTML::admin_name = 'GELOS administrator';


package Display::PLAIN;

$Display::PLAIN::debug = $Display::debug
   if !$Display::PLAIN::debug;

sub Display::PLAIN::Record {
   my ($record, $indent) = @_;
   my $rmsg = '';

   if ( ref($record) eq 'ARRAY' ) {
      my $i;
      for ( $i = 0; $i <= $#{ $record }; $i++) {
         $rmsg .= &Record(${ $record }[$i], $indent);
      }
   } elsif (ref($record) eq 'HASH' or ref($record) eq 'SCALAR' or !ref($record) ) {
      my ($label, $value, $errormsg) = ('','','');
      if ( !ref($record) ) {
         $value = $record;
      } elsif ( ref($record) eq 'SCALAR' ) {
         $value = $$record;
      } else {
         $label = $record->{label};
         $value = $record->{value};
         $errormsg = $record->{error};
      }

      if ( $label ) {
         $label =~ s/\s+$//;              # zap trailing whitespace
         $label =~ s/\t/        /g;       # convert tabs into spaces
         $label = (' 'x$indent) . ($indent? ';':'') . $label;
      }

      if ( $value ) {
         $value = $record->{value};
         my $spacer = ' 'x($indent+4);
         $value =~ s/[\n\s]+$//;          # remove trailing newlines and whitespace
         $label =~ s/\t/        /g;       # convert tabs into spaces
         $value =~ tr/\x00-\x1f/./;       # remove control characters
         $value .= "\n" . '(' . $record->{error} . ')'
            if defined($errormsg);
         $value =~ s/\n/\n$spacer/gs;
         $value = $spacer . $value;
      } elsif ( $errormsg ) {
         $value = $errormsg;
      } else {
         $value = ' ';
      }

      if ( $label or $value) {
         $rmsg = $label . "\n";
         $rmsg .= $value . "\n"
            if $value;
      }

      $rmsg .= &Record($record->{ref}, $indent+4)
         if $record->{ref};

   } else {
      warn "-- Cannot handle references of type '" . ref($record) . "'"
         if $Display::SUTRS::debug;
      $rmsg = (' 'x$indent) . 'Error in displaying record' . "\n";
   }

   return $rmsg;
}

sub Display::PLAIN::Error {
   my (@errormsgs) = @_;
   my $errormsg;

   # Find the first msg that is not empty
   #
   foreach $errormsg ( @errormsgs, 'Error occurred') {
      return $errormsg
         if $errormsg;
   }
}

sub Display::PLAIN::Header {
   my ($title) = @_;

   return ' === ' . $title . ' === ';
}

sub Display::PLAIN::Footer {
}
   
1;


package Display::HTML;

$Display::HTML::debug = $Display::debug
   if !$Display::HTML::debug;


%Display::HTML::errortext = (
   200=>'Ok',
   500=>'Internal Server Error'
);

sub Display::HTML::isHTML {
   my ($value) = @_;

   # this routine is one big hack. Only use it if there is no better way to
   # figure out the truth about the content of the value.

   return 1
      # Should be ok in almost all cases.
      if $value =~ m/^<(PRE|FRAGMENT|B|I|LI|HTML|BODY|TABLE|TR)/i
      or $value =~ m/<A[\s\n]+HREF=/s
      # Moderately accurate
      or $value =~ m/<(FRAGMENT|LI|HTML|BODY|TABLE|TR)>/i
      # pure guessing
      or $value =~ m/<(FRAGMENT|HTML|BODY|TABLE)[\s\n]/si;

   return 0;
}

# Creates proper HTML out of a plain text thingy
#
# It takes a guess at whether the text already contains HTML
#
sub Display::HTML::htmlify {
   my (@txt) = @_;

   foreach (@txt) {
      if ( &isHTML($_) ) {
         s#([\w\d.+-]+@[\w\d+-]+(?:\.[\w\d+-]+)+)(?!>)#<a href="mailto:\1">\1</a>#sg;
         s#([a-zA-Z]+://[^\s"'><]+)#<a href="\1">\1</a>#sg;
      } else {
         s/&/&amp;/sg;
         s/>/&gt;/sg;
         s/</&lt;/sg;
         s/[ \t]*\n/<BR>\n/sg;
         if ( ! m/<a[\n\s]+href/i ) {
            s#([\w\d.+-]+@[\w\d+-]+(?:\.[\w\d+-]+)+)#<a href="mailto:\1">\1</a>#sg;
            s#([a-zA-Z]+://\S*?)(&amp;|&gt;|&lt;|'|")#<a href="\1">\1</a>#sg;
         }
      }
      # The value $_ implicitly refers to an element of @txt, so @txt is
      # implicitly modified. Sneaky, but it works.
   }

   return (wantarray? @txt : $txt[0]);
}


sub Display::HTML::Record {
   my ($record, $indent) = @_;
   my ($label, $value, $errormsg) = ();
   my ($i, $n);
   my $rmsg;

   if ( ref($record) eq 'ARRAY' ) {
      for ( $i = 0; $i <= $#{ $record }; $i++) {
         $rmsg .= &Record(${ $record }[$i], $indent);
      }
   } elsif (ref($record) eq 'HASH' or ref($record) eq 'SCALAR' or !ref($record) ) {
      if ( !ref($record) ) {
         $value = $record;
      } elsif ( ref($record) eq 'SCALAR' ) {
         $value = $$record;
      } else {
         $label = $record->{label};
         $value = $record->{value};
         $errormsg = $record->{error};
      }

      if ( $label ) {
         $label =~ s/^\n+//;              # remove leading newlines
         $label =~ s/[\n\s]+$//;          # zap trailing whitespace
         $label =~ s/\t/        /g;       # convert tabs into spaces
         $label =~ s/ /&nbsp;/g;          # convert spaces into a non-breakables
         $label = ('&nbsp;'x$indent) . $label;
      }

      if ( $value ) {
         $value =~ s/^\n+//;              # remove leading newlines
         $value =~ s/[\n\s]+$//;          # remove trailing newlines and whitespace
         $value =~ s/\t/        /g;       # convert tabs into spaces
         $value =~ tr/\x00-\x09\x0b-\x1f/./;       # remove control characters
         $value = &htmlify($value);
      }

      if ( $errormsg and $value ) {
         $value .= ' (<FONT COLOR=red>' . $errormsg . '</FONT>)';
      } elsif ( $errormsg ) {
         $value = '<FONT COLOR=red>' . $errormsg . '</FONT>';
      }

      if ( $label or $value ) {
         $label = '&nbsp;'                # avoid empty cells in netscape
            if !$label;

         if ( $value ) {
            $rmsg = '<TR VALIGN=TOP><TD>' . $label . '</TD><TD>' . $value . '</TD></TR>' . "\n";
         } else {
            $rmsg = '<TR><TD COLSPAN=2>' . $label . '</TD></TR>' . "\n";
         }
      }

      $rmsg .= &Record($record->{ref}, $indent+4)
         if ref($record) eq 'HASH' and $record->{ref};
   } else {
      warn "-- Cannot handle references of type '" . ref($record) . "'"
         if $Display::HTML::debug;
      $rmsg = '<TR><TD>&nbsp</TD><TD><FONT COLOR=red>Error in displaying record</FONT></TD></TR>' . "\n";
   }

   return $rmsg;
}


sub Display::HTML::Error {
   my (@errormsgs) = @_;
   my $errormsg;

   # Find the first msg that is not empty
   #
   foreach $errormsg ( @errormsgs, 'Error occurred' ) {
      if ($errormsg) {
         return '<table><tr>'
              . '<td align=center>An error occurred during execution of your query:<p>'
              . '<font color="red" size="+3">'
              . $errormsg
              . '</font></td></tr></table>';
      }
   }
}


sub Display::HTML::Header {
   my ($title) = @_;

   $title = 'PerlZ: ' . $title
      if $title !~ m/PerlZ/i;;
   my ($tmsg, $hmsg) = ();
   
   if ($title) {
      $tmsg = '<head><title>'.$title.'</title></head>';
      $hmsg = '<hr><h2>'.$title.'</h2><hr>';
   }

   return '<html>'.$tmsg.'<body bgcolor="white">'.$hmsg;
}


sub Display::HTML::Footer {
   my $rmsg = '';

   $rmsg = join('', '<hr><center><a href="mailto:',
                    $Display::HTML::admin_email,
                    '">',
                    $Display::HTML::admin_name,
                    '</a></center>')
      if $Display::HTML::admin_email and $Display::HTML::admin_name;
   return $rmsg . '</body></html>' . "\n";
}


sub Display::HTML::Page {
   my ($errorcode, $title, $content, $footer) = @_;

   $content = $title
      if !$content;

   my (%headers) = ();

   $headers{Status} = $errorcode.' '.$Display::HTML::errortext{$errorcode}
      if $errorcode != 200;
   $headers{'Content-type'} = 'text/html';


   return join("\r\n", map { $_.': '.$headers{$_} } keys %headers)
        . "\r\n\r\n"
        . &Display::HTML::Header($title)
        . $content
        . &Display::HTML::Footer();
}

sub Display::HTML::ErrorPage {
   my ($errorcode, @errormsgs) = @_;
   my $errormsg;

   push @errormsgs, 'Undefined error'
      if !@errormsgs;

   foreach $errormsg (@errormsgs) {
      return &Display::HTML::Page($errorcode, $errormsg,
                                  &Display::HTML::Error(@errormsgs))
         if $errormsg;
   }
}

1;


package Display::HGS;

$Display::HGS::debug = $Display::debug
   if !$Display::HGS::debug;

sub Display::HGS::Record {
   my ($record, $uri) = @_;
   my $title = '';
   my ($description, $element, $rmsg) = ('', '', '');
   my @uri = ();

   push @uri, $uri
      if $uri;

   if ( ref($record) eq 'ARRAY' ) {
      foreach $element ( @{ $record } ) {
         if ( $element->{label} and $element->{value} ) {
            if (!$title and
                (  lc($element->{label}) eq 'title'
                or lc($element->{label}) eq 'uniform title'
                or lc($element->{label}) eq 'name' )) {
               $title = $element->{value};
            }
            if (!$description and
                (  lc($element->{label}) eq 'description'
                or lc($element->{label}) eq 'abstract')) {
               $description = $element->{value};
            }
            if (   lc($element->{label}) eq 'uri'
                or lc($element->{label}) eq 'url' ) {
               push @uri, $element->{value};
            }
         }
      }
   } elsif ( ref($record) eq 'HASH' ) {
      if ( !$record->{value} and $record->{ref} ) {
         return &Display::HGS::Record($record->{ref}, $uri);
      } else {
         $title = $record->{value};
      }
   } elsif ( ref($record) eq 'SCALAR' ) {
      $title = $$record;
   } elsif ( !ref($record) ) {
      $title = $record;
   } else {
      warn '-- What does a reference like ' . ref($record) . ' look like anyway?';
      return undef;
   }

   warn "-- No title"
      if $Display::debug and !$title;
   warn "-- No uri"
      if $Display::debug and !@uri;

   return undef                        # nothing sensible found really
      if !$title and $#uri == -1;

   $title = '(not title found in record)'
      if !$title;

   $rmsg = "Name: $title\n";
   $rmsg .= "Description: $description\n"
            if $description;
   map { $rmsg .= "URI: $_\n" } @uri;

   return $rmsg;
}


sub Display::HGS::Error {
   return '';
}


sub Display::HGS::Header {
   my ($name, $nresults, %headers) = @_;

   $nresults += 0;         # force it to be a numeric value

   $headers{Name} = $name
      if $name;

   return "EntriesExpected: $nresults\n"
        . "Version: 1.1\n"
        . "Engine: $::NAMESTRING, v$::VERSIONSTRING\n"
        . join("\n", map { $_.': '.$headers{$_} } keys %headers) . "\n";
        # FIXME no URI for server
}


sub Display::HGS::Footer {
   # just in case
   return '';
}


sub Display::HGS::Page {
   my ($name, $nresults, $content, %headers) = @_;

   my %header = ();

   return "Content-type: text/hgs\r\n\r\n"
        . &Display::HGS::Header($name, $nresults, %headers)
        . "\n"          # this empty line delimits the headers from the content
        . $content;
}


sub Display::HGS::ErrorPage {
   my ($name, @errormsgs) = @_;

   my $errormsg = "";

   # Find the first msg that is not empty
   #
   foreach $errormsg ( @errormsgs, 'Error occurred' ) {
      $errormsg =~ s/[\n\r\s]+/ /sg;      # convert the message to a single line
      return &Display::HGS::Page($name, 0, undef, ('Error'=>$errormsg))
         if $errormsg;
   }
}

1;
