# * ====================================================================
# * Copyright (c) 1995,1996, ISIS/JRC.  All rights reserved.
# *
# * This product was developed within the Institute for Safety, Informatics
# * and Systems of the Joint Research Centre of the European Commission.
# *
# * More information about us; and this software product can
# * be found on http://www.jrc.it.
# * 
# * With this file you should have received a file called 'License'.
# * 
# * Redistribution and use in source and binary forms, with or without
# * modification, are permitted provided that the following conditions
# * are met:
# *
# * 1. Redistributions of source code must retain the above copyright
# *    notice, this list of conditions and the following disclaimer. 
# *
# * 2. Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in
# *    the documentation and/or other materials provided with the
# *    distribution.
# *
# * 3. All materials mentioning features or use of this
# *    software must display the following acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * 4. The name of the Copyright holder must not be used to
# *    endorse or promote products derived from this software without
# *    prior written permission.
# *
# * 5. Redistributions of any form whatsoever must retain the following
# *    acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * THIS SOFTWARE IS PROVIDED BY ISIS/JRC ``AS IS'' AND ANY
# * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ISIS/JRC OR
# * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# * OF THE POSSIBILITY OF SUCH DAMAGE.
# * ====================================================================
#

# Read and process the PerlZ configuration files
#

package PerlZ;

# Global variables
#
$PerlZ::CONFIGFILENAME = 'perlz.conf';
$PerlZ::mysql = 'mysql';
$PerlZ::msql =  'mSQL';
$PerlZ::DBtype = $PerlZ::msql;
$PerlZ::DBtype = $PerlZ::mysql;

1;


package PerlZ::ConfigFile;

use strict;

use ConfigFile;
use Z3950;
use Z3950::Z3950;


# Verify that a URL starts with a /
#
sub PerlZ::ConfigFile::CheckURL {
   my ($key, $value) = @_;

   if ( ref $value ) {
      return (undef, 'url should not be a reference to '.ref($value));
   }

   if ( $value !~ m#^/# ) {
      return (undef, 'url should start with a slash');
   } elsif ( $value =~ m#/$# ) {
      return (undef, 'url cannot end in a slash');
   }

   return ($key, $value);
}

# Structure of the input files
#
# The main configuration file
#
%PerlZ::ConfigFile::Main = (
   'hostname'     => [],
   'port'         => [\&ConfigFile::Number],
   'socketname'   => [],
   'serversdir'   => [\&ConfigFile::ExistingDir],
   'urlroot'      => [\&PerlZ::ConfigFile::CheckURL],
   'docroot'      => [\&ConfigFile::ExistingDir],
   'docurlroot'   => [\&PerlZ::ConfigFile::CheckURL],
   'logfile'      => [\&ConfigFile::WritableFile],
   'templatefiles'=> [\&ConfigFile::CommaArray],
   'maxresults'   => [],
   'lifetime'     => [],
   'timeout'      => [],
   'dbase'        => [],
   'category'     => [\&ConfigFile::Lowercase,
                      \%PerlZ::ConfigFile::Category],
   'ID'           => 'Main',
   'DEFAULT'      => {'hostname'   => 'perlz.jrc.it',
                      'port'       => 2100,
                      'httpport'   => 80,
                      'urlroot'    => '/PerlZ',
                      'docroot'    => './docroot',
                      'docurlroot' => '/perlz',
                      'lifetime'   => 300,
                      'timeout'    => 300,
                      'logfile'    => 'logs/perlz.log',
                      'dbase'  => 'PerlZ',
                      'maxresults' => 100
                     }
);

%PerlZ::ConfigFile::Category = (
   'title'           => [],
   'description'     => [],
   'ID'              => 'Category',
   'DEFAULT'         => {'description' => 'No description'
                        }
);

1; # End of package PerlZ::ConfigFile

