#!/usr/local/bin/perl
# * ====================================================================
# * Copyright (c) 1995,1996, ISIS/JRC.  All rights reserved.
# *
# * This product was developed within the Institute for Safety, Informatics
# * and Systems of the Joint Research Centre of the European Commission.
# *
# * More information about us; and this software product can
# * be found on http://www.jrc.it.
# * 
# * With this file you should have received a file called 'License'.
# * 
# * Redistribution and use in source and binary forms, with or without
# * modification, are permitted provided that the following conditions
# * are met:
# *
# * 1. Redistributions of source code must retain the above copyright
# *    notice, this list of conditions and the following disclaimer. 
# *
# * 2. Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in
# *    the documentation and/or other materials provided with the
# *    distribution.
# *
# * 3. All materials mentioning features or use of this
# *    software must display the following acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * 4. The name of the Copyright holder must not be used to
# *    endorse or promote products derived from this software without
# *    prior written permission.
# *
# * 5. Redistributions of any form whatsoever must retain the following
# *    acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * THIS SOFTWARE IS PROVIDED BY ISIS/JRC ``AS IS'' AND ANY
# * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ISIS/JRC OR
# * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# * OF THE POSSIBILITY OF SUCH DAMAGE.
# * ====================================================================
#

use strict;

use CGI qw/:standard/;

use Display;


# Read parameters from CGI headers
#
my $category = CGI::param('category');
my $server = CGI::param('server');
my $dbase = CGI::param('dbase');

require 'categories.pl'; 

my $selecturl = $::docurlroot . '/dbase_select.pl';
my $searchurl = $::docurlroot . '/search.pl';

unless ( $server and $dbase ) {
   print &Display::HTML::Page(200, 'No server specified',
         qq|Please go to <A HREF="$selecturl">this page</A> and select a
            category, server and database to query</A>|);
   exit 0;
}

# Create the selector for the dbase name
# 0 dbases -> error page, 1 dbase -> print dbase, 2 ->create selector
#
my $dbsel;

my $serv = $::servers->{$server};

my @dbases = keys %{ $serv->{dbase} };
if ( @dbases == 0 ) {
   print &Display::HTML::ErrorPage(200, 'No databases available for '.$serv->{title});
   exit 0;
} elsif ( !exists $serv->{dbase}->{$dbase} ) {
   my $db = $serv->{dbase}->{$dbase};
   print &Display::HTML::Page(200, 'Database not available',
         qq|The database $db->{title} is not available from $serv->{title}.|);
   exit 0;
} elsif ( @dbases == 1 ) {
   my $db = $serv->{dbase}->{$dbase};
   $dbsel = qq|   <INPUT TYPE=HIDDEN NAME=dbase VALUE="$dbase">$db->{title}|;
} else {
   $dbsel .= qq|   <SELECT NAME="dbase">\n|;
   foreach my $dbase1 ( sort { $serv->{dbase}->{$a}->{title} cmp $serv->{dbase}->{$b}->{title} } @dbases ) {
      my $c = ($dbase1 eq $dbase? ' SELECTED>':'>        ');
      $dbsel .= qq|      <OPTION VALUE="$dbase1"$c$serv->{dbase}->{$dbase1}->{title}\n|;
   }
   $dbsel .= qq|   </SELECT>\n|;
}


# Create the rest of the page
#
print &Display::HTML::Page(200, "Search $serv->{title}", qq|
<FORM ACTION="$::urlroot" METHOD=GET>
   <INPUT TYPE=HIDDEN NAME="protocol" VALUE="HTML">
   <INPUT TYPE=HIDDEN NAME="server" VALUE="$server">
   <TABLE>
   <TR><TD>$serv->{title}, $dbsel
   </TD></TR><TR><TD ALIGN=CENTER>

   <SELECT NAME="key1">
      <OPTION VALUE="Title">Title</A>
      <OPTION VALUE="Local number">Local number</A>
      <OPTION VALUE="Any" SELECTED>Any</A>
   </SELECT>
   <INPUT TYPE=HIDDEN NAME="op1" VALUE="equal">equals&nbsp;&nbsp;
   <INPUT NAME="value1" TYPE=TEXT SIZE=40 VALUE="environment">

   </TD></TR><TR><TD ALIGN=CENTER>
    
   or

   </TD></TR><TR><TD ALIGN=CENTER>

   <SELECT NAME="key2">
      <OPTION VALUE="Title">Title</A>
      <OPTION VALUE="Local number">Local number</A>
      <OPTION VALUE="Any" SELECTED>Any</A>
   </SELECT>
   <INPUT TYPE=HIDDEN NAME="op2" VALUE="equal">equals&nbsp;&nbsp;
   <INPUT NAME="value2" TYPE=TEXT SIZE=40 VALUE="oil">

   </TD></TR><TR><TD ALIGN=CENTER>
   <INPUT TYPE=SUBMIT VALUE="Search">
   </TD></TR></TABLE>
</FORM>
|);
exit 0;
