#!/usr/local/bin/perl -I/home/n_hibma/z39.50/PerlZ
# * ====================================================================
# * Copyright (c) 1995,1996, ISIS/JRC.  All rights reserved.
# *
# * This product was developed within the Institute for Safety, Informatics
# * and Systems of the Joint Research Centre of the European Commission.
# *
# * More information about us; and this software product can
# * be found on http://www.jrc.it.
# * 
# * With this file you should have received a file called 'License'.
# * 
# * Redistribution and use in source and binary forms, with or without
# * modification, are permitted provided that the following conditions
# * are met:
# *
# * 1. Redistributions of source code must retain the above copyright
# *    notice, this list of conditions and the following disclaimer. 
# *
# * 2. Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in
# *    the documentation and/or other materials provided with the
# *    distribution.
# *
# * 3. All materials mentioning features or use of this
# *    software must display the following acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * 4. The name of the Copyright holder must not be used to
# *    endorse or promote products derived from this software without
# *    prior written permission.
# *
# * 5. Redistributions of any form whatsoever must retain the following
# *    acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * THIS SOFTWARE IS PROVIDED BY ISIS/JRC ``AS IS'' AND ANY
# * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ISIS/JRC OR
# * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# * OF THE POSSIBILITY OF SUCH DAMAGE.
# * ====================================================================
#


# infix2postfix(string)
#     Converts an expression like '"a"&b|(c&!d)'
#     into the postfix expression (RPN): ab&cd!&|
#
#     allowed characters: & = and, | = or, () = brackets and the rest should
#     be included in quotes either a pair of single or double quotes
#
#     on error undef is returned;
#

# BNF of expression handled:
# _expr   = orterm { '|' orterm }
# orterm  = andterm { ( '&' | '&' '!') andterm
# andterm = '(' _expr ')' | '"' { [^"] } '"' | "'" { [^'] } "'"
#           | { [^\s&|!] }
#
# N.B.: '& !' is valid but '| !', one cannot handle this in Z3950.

sub infix2postfix {
   my ($expr) = join(" ", @_);

   my ($PDU, $tail) = &Z3950::_expr($expr);

   warn "-- Left over after infix2postfix: '$tail'" 
      if $Z3950::debug and $tail;

   return $PDU;
}

sub _expr {
   my ($expr) = @_;
   my ($rt);

   my ($r, $tail) = &Z3950::orterm($expr);
   return ()                           # bail out on error
      if !$r;

   while ( $tail =~ s/^\s*\|\s*// ) {
      ($rt, $tail) = &Z3950::orterm($tail);

      return ()                        # bail out on error
         if !$rt;

      $r = &Z3950::ContSeq(1, $rt, $r,
                     &Z3950::ContVal(1, 0, $BER::NULL)   # OR sequence
           );
   }

   return ($r, $tail);
}

sub orterm {
   my ($expr) = @_;
   my ($rt);

   my ($r, $tail) = &Z3950::andterm($expr);
   return ()                           # bail out on error
      if !$r;

   while ( $tail =~ s/^\s*&\s*(!?)\s*// ) {
      ($rt, $tail) = &Z3950::andterm($tail);
      return ()                        # bail out on error
         if !$rt;

      $r = &Z3950::ContSeq(46,         # AND-NOT or AND sequence
              &Z3950::ContSeq(1, $r, $rt,
                 &Z3950::ContVal(($1 eq '!'? 2:0), 0, $BER::NULL)
              )
           );
   }

   return ($r, $tail);
}

sub andterm {
   my ($expr) = @_;

   if ( $expr =~ m/^\s*\(\s*(.*)/ ) {
      my ($expr, $tail) = &Z3950::find_corresponding_bracket($1);
      my ($r, $tail_expr) = &Z3950::_expr($expr);

      warn "'$tail_expr' ignored"
         if $Z3950::debug and $tail_expr;

      return ($r, $tail);

   } elsif (   $expr =~ m/^\s*(['"])(.*?)\1\s*(.*)/
            or $expr =~ m/^(\s*)([^\s|!&]+)\s*(.*)/ ) {
      my ($expr, $tail) = ($2, $3);

      # FIXME field should be detected based on the term
      my $field = 1016;

      my $r = &Z3950::ContSeq(0,                              # op
                 &Z3950::ContSeq(102,                        # attr term
                    &Z3950::ContSeq(44,                      # attributes
                       &Z3950::UnivSeq(
                          &Z3950::ContVal(120,      1, $BER::INTEGER), # Use attribute type
                          &Z3950::ContVal(121, $field, $BER::INTEGER)  # field number
                       )
                    ),
                    &Z3950::ContVal(45, $expr, $BER::OCTET_STR)        # search term
                 )
              );
      return ($r, $tail);
   } else {
      warn "Failed to decode '$expr'";
      return ();
   }
}


sub find_corresponding_bracket {
   my ($expr) = @_;
   my $c;                              # current character
   my $cnt = 0;                        # keeps track of the 'open' brackets
   my $i = 0;                          # current position in $expr

   while ($c = substr($expr, $i++, 1) ) {
      last
         if $c eq ')' and $cnt-- == 0;
      $cnt++
         if $c eq '(';

      $i = (index $expr, $c, $i+1)+1   # Find the corresponding quote
         if $c eq '"' or $c eq "'";
   }

   return (substr($expr, 0, $i-1), substr($expr, $i));
}
