#!/usr/local/bin/perl -I/home/n_hibma/z39.50/PerlZ
# * ====================================================================
# * Copyright (c) 1995,1996, ISIS/JRC.  All rights reserved.
# *
# * This product was developed within the Institute for Safety, Informatics
# * and Systems of the Joint Research Centre of the European Commission.
# *
# * More information about us; and this software product can
# * be found on http://www.jrc.it.
# * 
# * With this file you should have received a file called 'License'.
# * 
# * Redistribution and use in source and binary forms, with or without
# * modification, are permitted provided that the following conditions
# * are met:
# *
# * 1. Redistributions of source code must retain the above copyright
# *    notice, this list of conditions and the following disclaimer. 
# *
# * 2. Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in
# *    the documentation and/or other materials provided with the
# *    distribution.
# *
# * 3. All materials mentioning features or use of this
# *    software must display the following acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * 4. The name of the Copyright holder must not be used to
# *    endorse or promote products derived from this software without
# *    prior written permission.
# *
# * 5. Redistributions of any form whatsoever must retain the following
# *    acknowledgment:
# *    "This product includes software developed by ISIS/JRC"
# *
# * THIS SOFTWARE IS PROVIDED BY ISIS/JRC ``AS IS'' AND ANY
# * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ISIS/JRC OR
# * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# * OF THE POSSIBILITY OF SUCH DAMAGE.
# * ====================================================================
#

# recreate the categories.pl file in the docroot, containing the categories
# in a Perl format. This avoids the overhead of loading in all the modules
# required to read the config files, together with their dependencies.

use strict;

use Data::Dumper;

use PerlZ;
use ServerConfig;
use ConfigFile;

$::verbose = 0;                  # verbose mode

$::categories = ();

my @files = ();                  # server description files to process

my $arg;
while ( $arg = shift @ARGV ) {
   if ( $arg eq '-c' and $PerlZ::CONFIGFILENAME = shift @ARGV ) {
      # nop
   } elsif ( $arg eq '-v' ) {
      $::verbose = 1;
   } elsif ( $arg =~ m/^-/ ) {
      if ( $arg eq '-h' ) {
         &usage();
      } else {
         &usage('Invalid argument: '.join(', ', $arg, @ARGV));
      }
   } elsif ( $arg =~ /\S/ ) {
      &usage("Invalid server config filename '$arg'")
         if $arg =~ m#/#;
      push @files, $arg;
      warn "Added file '$arg'"
         if $::verbose;
   }
}

# Read configuration file
#
$::config = &ConfigFile::Read($PerlZ::CONFIGFILENAME,
                              \%PerlZ::ConfigFile::Main)
   or die "Could not read config file $PerlZ::CONFIGFILENAME";


warn "Combining files from $::config->{serversdir}/ into $::config->{docroot}/categories.pl\n";

# Fetch the files from the serversdir if none specified.
# Else, if one or more was specified, prepend the serversdir path.
#
@files = map { $::config->{serversdir}.'/'.$_ } @files
   if @files;
@files = glob($::config->{serversdir}.'/*')
   unless @files;

die "No files to be processed\n"
   if !@files;

# iterate over all the server description files
#
my $file;
my %serverconfig;

FILE: foreach $file (@files) {
   # filenames are assumed to be relative to the serversdir directory


   next                 # skip the README
      if $file eq $::config->{serversdir}.'/README';
   next                 # skip backup files
      if $file =~ m/[%~]$/;

   next                 # skip directories
      if -d $file;

   warn "Reading $file ...\n"
      if $::verbose;

   # Translate the name of the file to the name of the server (basename)
   #
   my $dbase;
   my $server = $file;
   $server =~ s#.*/##;

   # Read in the configuration file for the server
   #
   $serverconfig{$server} = &ConfigFile::Read($file, \%ServerConfig::Server)
      or next FILE;

   # Check whether there are actually any databases available on that server
   #
   if ( !keys %{ $serverconfig{$server}->{dbase} } ) {
      warn "Ignored $file (no databases)"
         if $::verbose;
      next FILE;
   }

   # Server categories
   my $category;
   foreach $category ( @{ $serverconfig{$server}->{categories} } ) {
      if ( exists $::config->{category}->{lc($category)} ) {
         &AddToCategory($category, $server, keys %{ $serverconfig{$server}->{dbase} });
      }
   }
   foreach $dbase ( keys %{ $serverconfig{$server}->{dbase} } ) {
      foreach $category ( @{ $serverconfig{$server}->{dbase}->{$dbase}->{categories} } ) {
         if ( exists $::config->{category}->{lc($category)} ) {
            &AddToCategory($category, $server, $dbase);
         }
      }
   }
}


# Open the destination file
#
my $outputfilename = "$::config->{docroot}/categories.pl";
open CAT, ">$outputfilename.$$"
   or die "Could not open $::config->{docroot}/categories.pl for writing, $!";

print CAT "# This file has been automatically created by\n";
print CAT "#   $0\n";
print CAT "# on\n";
print CAT "#   ".localtime(time())."\n";
print CAT "# \n";
print CAT "# The copyright displayed in that file applies to this file.\n";
print CAT "# The copyright displayed in the individual server description files\n";
print CAT "# apply to the information extracted from these files.\n";
print CAT "# \n\n";

# Dump the descriptions of the servers first, followed
# by the category info. This way we get a very neat and readable
# file.

$Data::Dumper::Indent = 1;
$Data::Dumper::Varname = '::VAR';
print CAT Dumper($::servers, $::categories);
print CAT '$::servers = $::VAR1;'."\n";
print CAT '$::categories = $::VAR2;'."\n";
print CAT "\n";
print CAT "\$::docurlroot = '$::config->{docurlroot}';\n";
print CAT "\$::urlroot = '$::config->{urlroot}';\n";

close CAT;

# And move the new file into its place (atomically)
my $rc;
$rc = system("mv $outputfilename.$$ $outputfilename")
   and warn "mv $outputfilename.$$ $outputfilename failed, $rc";



sub AddToCategory {
   my ($category, $server, @dbases) = @_;

   $::categories->{lc($category)}->{title} = $category
      unless $::categories->{lc($category)};

   $category = lc($category);

   $::categories->{$category}->{title} = $::config->{category}->{$category}->{title}
      if $::config->{category}->{$category}->{title};
   $::categories->{$category}->{description} = $::config->{category}->{$category}->{description}
      if $::config->{category}->{$category}->{description};

   $::servers->{$server}->{configfile}
            = $::config->{serversdir}.'/'.$server;
   $::servers->{$server}->{title}
            = $serverconfig{$server}->{title} || $server;
   $::servers->{$server}->{description}
            = $serverconfig{$server}->{description}
      if $serverconfig{$server}->{description};

   my $dbase;
   foreach $dbase ( @dbases ) {
      print "- " . $server . '/' . $dbase . ' -> ' . $category . "\n"
         if $::verbose
            and !$::categories->{$category}->{server}->{$server}->{dbase}->{$dbase};

      $::servers->{$server}->{dbase}->{$dbase}->{title}
               = $serverconfig{$server}->{dbase}->{$dbase}->{title} || $dbase;
      $::servers->{$server}->{dbase}->{$dbase}->{description}
               = $serverconfig{$server}->{dbase}->{$dbase}->{description}
         if $serverconfig{$server}->{dbase}->{$dbase}->{description};

      $::categories->{$category}->{server}->{$server} = $::servers->{$server};
   }
}


sub usage {
   my ($message) = @_;

   warn $message . ($message =~ m/\n$/s? '':"\n")
      if $message;

   die "Usage: $0 [-c <config file>] [-h] [-v]\n"
     . "   -h     help text\n"
     . "   -v     verbose output\n";
}
